from __future__ import absolute_import, division, print_function
import inspect
import sys
from builtins import (str)
from terraform_validator.custom_rules.BaseRule import BaseRule

def lineno():
    """Returns the current line number in our program."""
    return str(' - ManagedPolicyOnUserRule - caller: '+str(inspect.stack()[1][3])+' - line number: '+str(inspect.currentframe().f_back.f_lineno))



class ManagedPolicyOnUserRule(BaseRule):
  
  
  def __init__(self, cfn_model=None, debug=None):
    """
    Initialize
    :param cfn_model: 
    """
    BaseRule.__init__(self, cfn_model, debug=debug)

  def rule_text(self):
    """
    Get rule text
    :return: 
    """
    if self.debug:
      print('rule_text'+lineno())
    return 'IAM managed policy should not apply directly to users.  Should be on group'


  def rule_type(self):
    """
    Get rule type
    :return: 
    """
    self.type= 'VIOLATION::FAILING_VIOLATION'
    return 'VIOLATION::FAILING_VIOLATION'

  def rule_id(self):
    """
    Get rule id
    :return: 
    """
    if self.debug:
      print('rule_id'+lineno())
    self.id ='F12'
    return 'F12'


  def audit_impl(self):
    """
    Audit
    :return: violations 
    """
    if self.debug:
      print('ManagedPolicyOnUserRule - audit_impl'+lineno())

    violating_policies = []

    resources= self.cfn_model.resources_by_type('AWS::IAM::ManagedPolicy')

    if len(resources)>0:
      for resource in resources:
          if self.debug:
            print('resource: '+str(resource)+lineno())

          if hasattr(resource,'users'):
            if resource.users:
              if self.debug:
                print('users: '+str(resource.users))
              if len(resource.users)>0:

                violating_policies.append(str(resource.logical_resource_id))

    else:
      if self.debug:
        print('no violating_policies' + lineno())

    if self.debug:
      input('Press enter to continue: '+lineno())

    return violating_policies