type: map
mapping:

  variable:
    type: map
    required: no
    mapping:
      # insert parmaetrs here

  resource:
    type: map
    required: no
    mapping:
     # insert resource

  =:
    type: any